clear all
close all
clc

load yankees.txt

year = yankees(:,1);
PCT = yankees(:,2); %winning percentage in year t
OBP = yankees(:,5); %team on-base percentage in year t
SLG = yankees(:,6); %team slugging average in year t
ERA = yankees(:,8); %team earned run average in year t


tic

%Odhady Gibbsovym vzorkovacem bez predpokladu autokorelace

y = PCT;
X = [ones(size(y)) OBP SLG ERA];

[n,k] = size(X);

%neinformativni priory pro beta a h

nu0 = 0;
invV0 = zeros(k);

S = 30000;
S0 = 5000;
S1 = S-S0;


b_Gibbs = zeros(k,S);
h_Gibbs = zeros(1,S);


%Gibbsuv vzorkovac
h_Gibbs0 = 1;
V1 = inv(h_Gibbs0*X'*X);
b1 = V1*(h_Gibbs0*X'*y);
b_Gibbs(:,1)=b1+norm_rnd(V1);
%b_Gibbs(:,1)=mvnrnd(b1,V1)';

nu1 =n;
s12 = 1/nu1*((y-X*b_Gibbs(:,1))'*(y-X*b_Gibbs(:,1)));
h_Gibbs(:,1)=gamm_rnd(1,1,.5*nu1,.5*nu1*s12);

for i=2:S
 V1 = inv(h_Gibbs(:,i-1)*X'*X);
 b1 = V1*(h_Gibbs(:,i-1)*X'*y);
 %(nesystematicky zasah -- nekdy nastava numericky problem, ze Matlab nevezme V1 jako pozitivne
 %definitni matici => ignorujeme chybu a "udelame z V1 pozitivne definitni
 %prictenim maleho cisla 
 %try
 b_Gibbs(:,i)=b1+norm_rnd(V1);
 %catch
 %b_Gibbs(:,i)=b1+norm_rnd(V1+0.000000001);
 %end

nu1 =n;
s12 = 1/nu1*((y-X*b_Gibbs(:,i))'*(y-X*b_Gibbs(:,i)));
h_Gibbs(:,i)=gamm_rnd(1,1,.5*nu1,.5*nu1*s12);
end

%vyhozeni prvnich S0 vzorku
b_S1 = b_Gibbs(:,S0+1:S);
h_S1 = h_Gibbs(:,S0+1:S);


b_mean = mean(b_S1,2); %posteriorni stredni hodnota 
h_mean = mean(h_S1);
b_var = mean(b_S1.^2,2)-b_mean.^2; %posteriorni rozptyl
h_var = mean(h_S1.^2)-h_mean.^2;


%KONVERGENCNI DIAGNOSTIKY - Gewekova CD pomoci funkce momentg
alldraws = [b_S1' h_S1'];
%The function momentg is taken from LeSage's toolbox
%it inputs all Gibbs draws and produces posterior
%mean, standard deviation, nse and rne
%it calculates what the book calls S(0) in various ways
%see momentg.m for more details
result = momentg(alldraws);
means=[result.pmean]';
stdevs=[result.pstd]';
nse=[result.nse]';
nse1=[result.nse1]';
nse2=[result.nse2]';
nse3=[result.nse3]';
%calculate Geweke convergence diagnostic based on first .1
%and last .4 of draws
idraw1= round(.1*S1); %prvnich 10%
result = momentg(alldraws(1:idraw1,:));
meansa=[result.pmean]';
nsea=[result.nse1]';

idraw2= round(.6*S1)+1; %poslednich 40%
result = momentg(alldraws(idraw2:S1,:));
meansb=[result.pmean]';
nseb=[result.nse1]';

CD = (meansa - meansb)./(nsea+nseb);



%HPDI pro beta
[k_b,l_b] = size(b_S1);

hperc=0.95; %HPDI
HPDIb_l = zeros(k_b,1);
HPDIb_u = zeros(k_b,1);
for j=1:k_b
    botperc=round((1-hperc)/2*l_b);
    topperc=round((1+hperc)/2*l_b);
    temp = sort(b_S1(j,:));
    HPDIb_u(j)=temp(topperc);
    HPDIb_l(j)=temp(botperc);
end


t = toc;

%Prezentace vysledku, grafy
fprintf('Prior and posterior means for beta (std. deviations in parentheses)\n');
fprintf('                Prior       Posterior      NSE         Geweke CD             %3.1f%% HPDI \n',hperc*100);
fprintf('   ------------------------------------\n');

figure

for i=1:k
fprintf('Beta%1.0f           neinf.   %12.4f   %10.4f   %10.4f   [%10.4f, %10.4f]\n',[i b_mean(i) nse(i) CD(i) HPDIb_l(i) HPDIb_u(i)]);
fprintf('                   (N.A.) (%12.4f)\n', sqrt(b_var(i)));

    [f,xi] = ksdensity(b_S1(i,:));
    subplot(3,2,i)
    plot(xi,f)
    xlabel(['\beta_' num2str(i)])
end

fprintf('   ------------------------------------\n');
fprintf('Number of replications: %12.4f total \n',S);
fprintf('                        %12.4f discarded \n',S0);
fprintf('   ------------------------------------\n');

fprintf('Time elapsed (s): %12.2f \n',t);
fprintf('***************************************************************************\n\n\n');
